@@ -58,8 +58,11 @@ GType font_view_model_get_type (void);
 GtkTreeModel * font_view_model_new (void);
 
 gboolean font_view_model_get_iter_for_face (FontViewModel *self,
-                                            FT_Face face,
-                                            GtkTreeIter *iter);
+                                            FT_Face        face,
+                                            GtkTreeIter   *iter);
+
+void     font_view_model_set_scale_factor  (FontViewModel *self,
+                                            gint           scale_factor);
 
 G_END_DECLS
 
